#!/usr/bin/perl -w

# $Id: reblog_upgrade.cgi,v 1.7 2005/05/24 21:01:27 mfrumin Exp $
#
# This file is part of Reblog: http://reblog.org
# Distributed under the Gnu Public License, see LICENSE
#
# Copyright 2004 Michael Frumin, Michal Migurski
# This is the Movable Type Plugin code to implement the reBlog functionality into Movable Type
#
# Developed by Michael Frumin (mfrumin AT eyebeam DOT org)
# Initially for Eyebeam's reBlog (http://www.eyebeam.org/reblog/)
#
# NOTE: This only works if you are using MySQL for the database backend for Movable Type
  
  
use strict;
use File::Spec;
use constant REFEED_GUID_PREFIX => 'tag:refeed/';

my($MT_DIR);
BEGIN {
    if (0 && $0 =~ m!(.*[/\\])!) {
        $MT_DIR = $1;
    } else {
	$MT_DIR = $ENV{SCRIPT_FILENAME};
    }

    # back up to directory just above 'plugins'
    my ($vol, $dir, $f) = File::Spec->splitpath($MT_DIR);
    my (@dirs) = File::Spec->splitdir($dir);
    while(pop(@dirs) ne 'plugins' && scalar(@dirs) > 0){};
    push(@dirs, "");
    $MT_DIR = File::Spec->join(@dirs);
    
    unshift @INC, $MT_DIR . 'lib';
    unshift @INC, $MT_DIR . 'extlib';
}

eval {
    use URI::URL;
    require MT::App::CMS;
    my $app = MT::App::CMS->new( Config => $MT_DIR . 'mt.cfg',
                                 Directory => $MT_DIR )
        or die MT::App::CMS->errstr;
    local $SIG{__WARN__} = sub { $app->trace($_[0]) };

    my $cgi = $app->{query};

    my ($auth) = $app->login();

    if(!(defined($auth) && $auth)) {
	print $cgi->redirect($app->path . "mt.cgi");
	exit;
    }

    if(!ref($auth)) {
	$auth = MT::Author->load($auth);
    }

    $app->send_http_header;

    use MT::Util qw( encode_html );

    my $action = $cgi->param("action") || 'upgrade_form';
    my $back_url = $cgi->param("back") || $ENV{HTTP_REFERER} || "";

    require MT::App::CMS;
    my $header = MT::App::CMS::build_page ($app, "header.tmpl", { author_name => $auth->username, no_breadcrumbs => 1 }) || "";
    my $footer = MT::App::CMS::build_page ($app, "footer.tmpl", {}) || "";

    print $header;

    my $back = ($back_url ? qq{&lt;&lt;&lt;&nbsp;<a href="} . encode_html($back_url) . qq{">go back from whence you came</a>} : "");

    my ($status, $version) = MT::Plugin::Reblog::check_reblog_status();
    my $curr_version = $MT::Plugin::Reblog::VERSION;

    if($status eq '-upgrade') {
	if($action eq 'upgrade_form') {
	    upgrade_form($version, $curr_version, $cgi);
	}
	elsif($action eq 'upgrade') {
	    do_upgrade($version, $curr_version, $cgi);
	}
    }
    else {
	print qq{Sorry, you are currently not in need of a reBlog upgrade.<BR>};
    }
    
    print qq{<BR>&lt;&lt;&lt;&nbsp;<a href="../mt.cgi">Back to Movable Type</a><BR>\n};
    print ($back ? "<BR>$back<BR>" : "");
    print $footer;
    
};
if ($@) {
    print STDERR "ERROR: $@\n";
    print "Content-Type: text/html\n\n";
    print "Got an error: <BR><PRE>" . encode_html($@) . "</PRE>";
}


sub upgrade_form {
    my ($from, $to, $cgi)  = @_;
    
# Guess where their reFeed is installed -- next to MT
    my $uri = new URI::URL("http://$ENV{HTTP_HOST}$ENV{REQUEST_URI}");
    my (@pathC) = $uri->path_components();
    pop(@pathC); #chop off script name
    pop(@pathC); #chop off 'plugins'
    pop(@pathC); #chop off MT directory
    push(@pathC, "refeed");
    $uri->path_components(@pathC);

    my $host = $uri->host;
    my $path = $uri->path;

    my $config_mgr = MT::ConfigMgr->instance;
    my $DEFAULT_REBLOG_DB = $config_mgr->get("Database");
    my $DEFAULT_REFEED_URL = "http://$host$path/";
    my $DEFAULT_REBLOG_ITEM_TABLE = "rf_items";
    my $DEFAULT_REBLOG_ITEM_SELECT_TABLE = "rf_items_select";
    my $DEFAULT_REBLOG_FEED_TABLE = "rf_feeds";
    my $DEFAULT_REBLOG_FEED_SELECT_TABLE = "rf_feeds_select";

    my $agg = ($from == 0.1 ? "Feed On Feeds" : "ReceSS");

    my $rf_install_url = "${DEFAULT_REFEED_URL}install.php";

    print qq{In order to upgrade your reBlog plugin from v$from to v$to, we need a little information from you.  Please be careful here.<BR><BR>
		 We <b>STRONGLY SUGGEST that you back up both your database for Movable Type and for $agg/reFeed before proceeding</b>.<BR><BR>
		 Also, make sure that you <b>upgrade the Feed Aggregator ($agg --> reFeed) <i>BEFORE</i> upgrading the reBlog MT plugin.</b><BR>If you have not done that upgrade already, you probably can do it here: <a target="_refeed_upgrade" href="$rf_install_url">$rf_install_url</a>
		 <BR><BR>
	     };
    
    print qq{<form action="reblog_upgrade.cgi" method="GET">
		 <input type="hidden" name="action" value="upgrade">
		 <table>
		 <TR><TD>Name of MySQL database that houses reFeed:</TD><TD><input type="text" name="refeed_db" value="$DEFAULT_REBLOG_DB"></TD></TR>
		 <TR><TD>URL of your installation of reFeed:</TD><TD><input size="50" type="text" name="refeed_url" value="$DEFAULT_REFEED_URL"></TD></TR>
		 <TR><TD>Name of the reFeed ITEM_TABLE in the above MySQL DB:</TD><TD><input type="text" name="refeed_item_table" value="$DEFAULT_REBLOG_ITEM_TABLE"></TD></TR>
		 <TR><TD>Name of the reFeed ITEM_SELECT_TABLE in the above MySQL DB:</TD><TD><input type="text" name="refeed_item_select_table" value="$DEFAULT_REBLOG_ITEM_SELECT_TABLE"></TD></TR>
	     };

    if($from == 0.1) {
	print qq{
	    <TR><TD>Name of the reFeed FEED_TABLE in the above MySQL DB:</TD><TD><input type="text" name="refeed_feed_table" value="$DEFAULT_REBLOG_FEED_TABLE"></TD></TR>
		<TR><TD>Name of the reFeed FEED_SELECT_TABLE in the above MySQL DB:</TD><TD><input type="text" name="refeed_feed_select_table" value="$DEFAULT_REBLOG_FEED_SELECT_TABLE"></TD></TR>
	    };		
    }

    print qq{
	<TR><TD colspan="2"><input type="Submit" name="submit" value="Upgrade my reBlog MT plugin"></TD></TR>
	    </table>
	};
    
    
}

sub do_upgrade {
    my ($from, $to, $cgi) = @_;

    my $driver = MT::Object->driver();
    my $dbh = $driver->{dbh};

    if(!check_form_vals($from, $to, $dbh, $cgi)) {
	return;
    }

    print "Upgrading from  v$from to v$to ...<BR>\n"; 

    my $res = 0;

    if($from == 0.1) {
	$res = do_upgrade_0_1($dbh, $cgi);
    }
    elsif($from == 0.9) {
	$res = do_upgrade_0_9($dbh, $cgi);
    }

    if($res) {
	print "Upgraded from v$from to v$to.<BR>\n";
	print "Please make sure to re-configure your Reblog Source Feed to get entries from the new reFeed installation.<BR>\n";
    }
    else {
	print "Sorry, but something went wrong, and the upgrade was rolled back.<BR>\n";
    }
}

sub check_form_vals {
    my ($from, $to, $dbh, $cgi) = @_;

    my (@keys) = qw(refeed_db refeed_item_table refeed_item_select_table refeed_url);
    if($from == 0.1) {
	push(@keys, qw(refeed_feed_table refeed_feed_select_table));
    }
    
    foreach my $k (@keys) {
	if(! $cgi->param($k)) {
	    print "PLEASE SPECIFY ALL OF THE VARIABLES IN THE UPGRADE FORM.<BR>\n";
	    return 0;
	}
    }


    my $res = 0;

    my $rf_db = $cgi->param('refeed_db');
    my $rf_ist = $cgi->param('refeed_item_select_table');

    my $RE = $dbh->{RaiseError};

    $dbh->{RaiseError} = 1;


    my $row = $dbh->selectrow_arrayref(qq{SHOW TABLES FROM `$rf_db` LIKE '$rf_ist'});
    if(defined($row)) {
	$res = 1;
    }
    else {
	print "Sorry, the database you specified, '$rf_db' , doesn't contain the reFeed tables.<BR>\n";
    }

    $dbh->{RaiseError} = $RE;
    
    return $res;
}

sub do_upgrade_0_1 {
    my ($dbh, $cgi) = @_;

    my $rf_db = $cgi->param('refeed_db');
    my $rf_it = $cgi->param('refeed_item_table');
    my $rf_ist = $cgi->param('refeed_item_select_table');
    my $rf_ft = $cgi->param('refeed_feed_table');
    my $rf_fst = $cgi->param('refeed_feed_select_table');
    my $rf_url = $cgi->param('refeed_url');


    my $ds = MT::ReblogData->datasource;
    my $e_ds = MT::Entry->datasource;

    my $config_mgr = MT::ConfigMgr->instance;
    my $mt_db = $config_mgr->get("Database");

    $rf_db = ($rf_db ne $mt_db ? "`$rf_db`." : "");

    print "Creating $ds table ...<BR>\n";

    print MT::Plugin::Reblog::create_reblog_data_table();

    print "Populating $ds table ...<BR>\n";

    my $sql = qq{INSERT INTO mt_$ds
		     (${ds}_entry_id 
		      , ${ds}_link 
		      , ${ds}_guid 
		      , ${ds}_source_author 
		      , ${ds}_via_link 
		      , ${ds}_orig_created_on 
		      , ${ds}_source 
		      , ${ds}_source_url 
		      , ${ds}_source_feed_url 
		      , ${ds}_created_on 
		      , ${ds}_modified_on 
		      , ${ds}_created_by 
		      , ${ds}_modified_by )
		     SELECT
		     e.${e_ds}_id
		     , e.entry_reblog_link
		     , md5(concat(if(f.url is null, "", f.url)
				  , e.entry_reblog_link
				  , e.entry_title
				  , e.entry_text)
			   )
		     , e.${e_ds}_reblog_author
		     , e.${e_ds}_reblog_link
		     , e.${e_ds}_created_on
		     , e.${e_ds}_reblog_source
		     , e.${e_ds}_reblog_source_link
		     , if(f.url is NULL, "", f.url)
		     , e.${e_ds}_created_on
		     , e.${e_ds}_modified_on
		     , e.${e_ds}_created_by
		     , e.${e_ds}_modified_by
		     FROM mt_$e_ds e
		     LEFT JOIN $rf_db`$rf_ft` f
		     ON e.${e_ds}_reblog_source_link = f.link
		     WHERE e.${e_ds}_reblog_link != '' AND e.${e_ds}_reblog_source != ''
		 };
    
    if(!$dbh->do($sql)) {
	print "<b>ERROR: " . $dbh->errstr . "</b><BR>\n";
    }

    if(update_guids($dbh, $cgi)) {
	$dbh->do("ALTER TABLE `$e_ds` DROP `entry_reblog_link`");
	$dbh->do("ALTER TABLE `$e_ds` DROP `entry_reblog_author`");
	$dbh->do("ALTER TABLE `$e_ds` DROP `entry_reblog_source`");
	$dbh->do("ALTER TABLE `$e_ds` DROP `entry_reblog_source_link`");
	return 1;
    }
    else {
	print "Unable to fix the GUID's for any of your reBlogged posts.  Rolling back this upgrade...<BR>\n";
	$dbh->do(qq{DROP TABLE mt_$ds });
	return 0;
    }
}

sub do_upgrade_0_9 {
    my ($dbh, $cgi) = @_;

    my ($ds_table, $ds_table_temp) = restructure_table($dbh);

    if(update_guids($dbh, $cgi)) {
	$dbh->do(qq{DROP TABLE $ds_table_temp});
	return 1;
    }
    else {
	print "Unable to fix the GUID's for any of your reBlogged posts.  Rolling back this upgrade...<BR>\n";
	$dbh->do(qq{DROP TABLE $ds_table });
	$dbh->do(qq{ALTER TABLE $ds_table_temp RENAME to $ds_table});
	return 0;
    }



}

sub restructure_table {
    my ($dbh) = @_;

    my $ds = MT::ReblogData->datasource;

    my $ds_table = 'mt_' . $ds;
    my $ds_table_temp = $ds_table . '__TEMP__';
    
    my $sql = qq{ALTER TABLE $ds_table RENAME to $ds_table_temp};
    $dbh->do($sql);
    
    print MT::Plugin::Reblog::create_reblog_data_table();
    
    print "Upgrading $ds table ...<BR>\n";
    
    $sql = qq{INSERT INTO $ds_table
		  (${ds}_id , ${ds}_entry_id , ${ds}_link , ${ds}_guid , ${ds}_source_author , ${ds}_via_link , ${ds}_orig_created_on , ${ds}_source , ${ds}_source_url , ${ds}_source_feed_url , ${ds}_created_on , ${ds}_modified_on , ${ds}_created_by ,${ds}_modified_by )
		  SELECT
		  ${ds}_id, ${ds}_entry_id, ${ds}_link, ${ds}_post_hash, ${ds}_author, ${ds}_link, ${ds}_created_on, ${ds}_source, ${ds}_source_link, ${ds}_source_link_xml, ${ds}_created_on, ${ds}_modified_on, ${ds}_created_by, ${ds}_modified_by
		  FROM $ds_table_temp
	      };
    
    $dbh->do($sql);

    return ($ds_table, $ds_table_temp);
}

sub update_guids {
    my ($dbh, $cgi) = @_;


    my $rf_db = $cgi->param('refeed_db');
    my $rf_it = $cgi->param('refeed_item_table');
    my $rf_ist = $cgi->param('refeed_item_select_table');
    my $rf_url = $cgi->param('refeed_url');

    my $rf_uri = new URI::URL($rf_url);
    my (@pathC) = $rf_uri->path_components;
    if($pathC[scalar(@pathC) - 1] =~ /\.php$/i) {
	pop(@pathC);
    }
    if($pathC[scalar(@pathC) - 1] =~ /^frames$/i) {
	pop(@pathC);
    }
    $rf_uri->path_components(@pathC);

    my $rf_host = $rf_uri->host;
    my $rf_path = $rf_uri->path;

    print "Attempting to fix the GUID's for your reBlog posts ...<BR>\n";

    my (@rbds) = MT::ReblogData->load();

    print "Found " . scalar(@rbds) . " reBlog datums<BR>\n";
    
    my $config_mgr = MT::ConfigMgr->instance;
    my $mt_db = $config_mgr->get("Database");
    $rf_db = ($rf_db ne $mt_db ? "`$rf_db`." : "");

    print "Looking in the " . ($rf_db ? $rf_db : $mt_db) . " database for GUID'S ...<BR>\n";

    my %rbd_hash;
    my @guids;

    foreach my $rbd (@rbds) {
	my $rf_guid = REFEED_GUID_PREFIX . $rbd->guid;
	$rbd_hash{$rf_guid} = $rbd;
	push(@guids, $rf_guid);
    }

    if(scalar(@guids) > 0) {
	my $rf_guids = join(',', map{"'$_'";} @guids);

	my $sql = qq{SELECT ist.`id` id, it.`guid` guid 
			 FROM  $rf_db`$rf_it` it
			 RIGHT JOIN $rf_db`$rf_ist` ist
			 ON it.`id` = ist.`item_id`
			 WHERE it.`guid` in ($rf_guids)
		     };

	my $res = $dbh->selectall_arrayref($sql);
	my $i = 0;

	foreach my $row (@$res) {
	    my ($id, $guid) = @$row;
	    my $rbd = $rbd_hash{$guid};
	    my $rf_guid = refeed_guid($rf_host, $rf_path, $id);
	    $rbd->guid($rf_guid);
	    $rbd->save();
	    $i++;
	}

	if($i > 0) {
	    print "Fixed the GUID for $i out of " . scalar(@guids) . " entries.  You're probably OK.<BR>\n";
	}

	return $i;
    }

    return 0;
}

sub refeed_guid {
    my ($rf_host, $rf_path, $rfid) = @_;

    return "tag:$rf_host$rf_path$rfid";
}


